import pytest
from pathlib import Path

from genologics_mock.lims import MockLims
from genologics_mock.entities import (
    MockArtifact,
    MockProcess,
    MockProcessType,
    MockSample,
)
from .helpers import Helpers


@pytest.fixture
def config():
    """Get file path to config"""

    return "tests/fixtures/config.yaml"


@pytest.fixture
def entety_json_data():
    """Get file path to entety yaml data"""

    return "tests/fixtures/entety_data.json"


@pytest.fixture
def lims():
    return MockLims()


@pytest.fixture
def sample():
    return MockSample()


@pytest.fixture
def artifact():
    return MockArtifact()


@pytest.fixture
def process():
    return MockProcess()


@pytest.fixture(name="helpers")
def fixture_helpers():
    """Return a class with small helper functions"""
    return Helpers()


@pytest.fixture
def enzymatic_fragmentation_file():
    """Get file path to valid json"""
    file_path = "tests/fixtures/Enzymatic_fragmentation"
    file = Path(file_path)
    return file.read_text()


@pytest.fixture
def kapa_library_preparation_file():
    """Get file path to valid json"""
    file_path = "tests/fixtures/KAPA_Library_Preparation"
    file = Path(file_path)
    return file.read_text()


@pytest.fixture
def kapa_library_preparation_file_missing_udf():
    """Get file path to valid json"""
    file_path = "tests/fixtures/KAPA_Library_Preparation_missing_udf"
    file = Path(file_path)
    return file.read_text()




#!/usr/bin/env python
from cg_lims import options

from genologics.lims import Lims

import click
import yaml

# commands
from cg_lims.EPPs import epps


@click.group(invoke_without_command=True)
@options.config()
@click.pass_context
def cli(ctx, config):

    with open(config) as file:
        config_data = yaml.load(file, Loader=yaml.FullLoader)
    lims = Lims(
        config_data["BASEURI"], config_data["USERNAME"], config_data["PASSWORD"]
    )

    ctx.ensure_object(dict)
    ctx.obj["lims"] = lims


cli.add_command(epps)



from genologics.entities import Artifact
from genologics.config import BASEURI
from genologics.lims import Lims

import pathlib
from typing import List


def get_log_content(lims: Lims, file_id: str) -> str:
    """Searching for a log file Artifact with file_id. 
    
    If the artifact is found, returning its file content."""

    log_artifact = Artifact(lims, id=file_id)

    try:
        files = log_artifact.files
    except:
        files = None

    if files:
        server_adress = BASEURI.split(":")[1]
        file_path = files[0].content_location.split(server_adress)[1]
        log_file = pathlib.Path(file_path)
        log_file_content = log_file.read_text()
    else:
        log_file_content = ''

    return log_file_content



  
"""Base cg_lims calss definitions
"""

from cg_lims.exceptions import ZeroReadsError, MissingUDFsError
from genologics.entities import Artifact


class Pool:
    """Collecting informatiuon general to any pool"""

    def __init__(self, pool_artifact: Artifact):
        self.total_reads_missing = 0
        self.artifact = pool_artifact

    def get_total_reads_missing(self)-> None:
        """Get the total numer of missing reads in the pool"""

        for art in self.artifact.input_artifact_list():
            reads = art.samples[0].udf.get('Reads missing (M)')
            if reads is None:
                raise MissingUDFsError('Missing udfs: Reads missing (M)')
            self.total_reads_missing += reads
        if self.total_reads_missing == 0:
            raise ZeroReadsError(
            "All samples seem to have Missing Reads = 0!"
        )





#!/usr/bin/env python

from cg_lims.get.artifacts import get_artifacts, get_latest_artifact
from cg_lims.exceptions import LimsError, MissingUDFsError
from cg_lims.files.make_csv import make_plate_file
from cg_lims import options
from genologics.lims import Lims
from genologics.entities import Process

import string
import logging
import click
import sys

LOG = logging.getLogger(__name__)


MINIMUM_AMOUNT = 10


def get_ligation_master_mix(amount):
    """Two different master mixes are used, A and B. 
    Master mix B has less volume of "xGen CS Adapter-Tech Access" 
    and is used for samples with DNA amount <25 ng.
    """

    if amount < 25:
        return 'B'
    else:
        return 'A'


def get_pcr_plate(amount):
    """Depending on amount of DNA the samples are run with different number 
    of PCR-cycles. These are run in three different PCR-plates. 
    The script sets the PCR-plate.
    """

    if amount < 50:
        plate = 'Plate3'
    elif amount < 200:
        plate = 'Plate2'
    else:
        plate = 'Plate1'
    return plate


def get_index_well(art):
    """Parsing out the index well position from the reagent label string wich 
    typically looks like this: 'A05 IDT_10nt_446 (AGCGTGACCT-CCATCCGAGT)'
    """

    if art.reagent_labels:
        # Assuming one reagent label per artifact (reagent_labels is a list):
        reagent_label = art.reagent_labels[0]

        # Getting the index well:
        index_well_with_zero = reagent_label.split(" ")[0]

        # Picking out column and removing zeros by int():
        index_well_col = int(index_well_with_zero[1:])
        index_well_row = index_well_with_zero[0]
        index_well = f"{index_well_row}{index_well_col}"
    else:
        index_well = "-"
    return index_well


def get_file_data_and_write(lims: Lims, amount_step: str, artifacts: list, file: str) -> dict:
    """Getting row data for hamilton file based on amount and reagent label"""

    failed_samples = []
    file_rows = {}
    for art in artifacts:
        sample_id = art.samples[0].id
        index_well = get_index_well(art)
        well = art.location[1].replace(":", "")
        amount_artifact = get_latest_artifact(lims, sample_id, amount_step)
        amount = amount_artifact.udf.get("Amount needed (ng)")

        if not amount:
            failed_samples.append(sample_id)
            continue

        if amount <= MINIMUM_AMOUNT:
            amount = MINIMUM_AMOUNT

        ligation_master_mix = get_ligation_master_mix(amount)
        pcr_plate = get_pcr_plate(amount)

        file_rows[well] = [
            sample_id,
            well,
            ligation_master_mix,
            index_well,
            pcr_plate,
        ]
        art.udf["Ligation Master Mix"] = ligation_master_mix
        art.udf["PCR Plate"] = pcr_plate
        art.put()

    headers = [
        "LIMS ID",
        "Sample Well",
        "Ligation Master Mix",
        "Index Well",
        "PCR Plate",
    ]

    make_plate_file(file, file_rows, headers)

    if failed_samples:
        raise MissingUDFsError(
            f"Could not find udf: Amount needed (ng) for samples: {', '.join(failed_samples)}, from step {amount_step}."
        )


@click.command()
@options.process_type(help="Amount step name.")
@options.file_placeholder(help="Hamilton file.")
@click.pass_context
def make_kapa_csv(ctx, file, process_type):
    """Script to make a csv file for hamilton. See AM doc #2125
    """
    LOG.info(f"Running {ctx.command_path} with params: {ctx.params}")
    process = ctx.obj["process"]
    lims = ctx.obj["lims"]
    artifacts = get_artifacts(process=process, input=False)

    file_name = f"{file}_{artifacts[0].container.name}_{process.type.name.replace(' ', '_')}"

    try:
        get_file_data_and_write(lims, process_type, artifacts, file_name)
        click.echo("The file was sucessfully generated.")
    except LimsError as e:
        sys.exit(e.message)



#!/usr/bin/env python

import click

# commands
from cg_lims.EPPs.udf.calculate.twist_pool import twist_pool
from cg_lims.EPPs.udf.calculate.twist_aliquot_amount import twist_aliquot_amount
from cg_lims.EPPs.udf.calculate.twist_aliquot_volume import twist_aliquot_volume


@click.group(invoke_without_command=True)
@click.pass_context
def calculate(ctx):
    """Main entry point of calculate commands"""
    pass


calculate.add_command(twist_pool)
calculate.add_command(twist_aliquot_amount)
calculate.add_command(twist_aliquot_volume)





from genologics.entities import Process, Artifact, Sample
from genologics.config import BASEURI
from genologics.lims import Lims

from cg_lims.get.ids import unique_list_of_ids
import logging
import pathlib
from typing import List
LOG = logging.getLogger(__name__)

from cg_lims.exceptions import QueueArtifactsError, MissingArtifactError




def queue_artifacts(lims: Lims, artifacts: List[Artifact], workflow_id: str, stage_id: str) -> None:
    """Queue artifacts to stage in workflow."""

    if not artifacts:
        LOG.warning("Failed trying to queue empty list of artifacts.")
        raise MissingArtifactError("No artifacts to queue.")
    stage_uri = (
        f"{BASEURI}/api/v2/configuration/workflows/{workflow_id}/stages/{stage_id}"
    )

    artifact_ids = unique_list_of_ids(artifacts)
    try:
        lims.route_artifacts(artifacts, stage_uri=stage_uri)
        LOG.info(f"Queueing artifacts to {stage_uri}.")
        LOG.info(
            f"The following artifacts have been queued: {' ,'.join(artifact_ids)}"
        )
    except:
        raise QueueArtifactsError("Failed to queue artifacts.")



#!/usr/bin/env python

from cg_lims import __version__ as version
from setuptools import setup, find_packages

try:
    with open("requirements.txt", "r") as f:
        install_requires = [x.strip() for x in f.readlines()]
except IOError:
    install_requires = []

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(name='cg_lims',
        version=version,
        description='Lims code for Clinical Genomics',
        long_description=long_description,
        long_description_content_type="text/markdown",
        author='Maya Brandi',
        url="https://github.com/Clinical-Genomics/cg_lims",
        author_email='maya.brandi@scilifelab.se',
        packages=find_packages(),
        include_package_data=True,
        entry_points={
        'console_scripts': ['lims=cg_lims.commands:cli'],
        },
        classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        ],
        python_requires='>=3.6',
     )



from typing import List
from copy import deepcopy
import json
from pathlib import Path


from genologics_mock.lims import MockLims
from genologics_mock.entities import (
    MockArtifact,
    MockProcess,
    MockProcessType,
    MockSample,
)


class Helpers:
    """Fixure Help functions to create fixures."""


    @staticmethod
    def read_json(file_path: str, key: str) -> list:
        with open(file_path) as json_file:
            data = json.load(json_file)
        return data[key]

    # Ensure methods: 
    # Creating enteties defined by arguments. 
    # Appending the new enteties to the given MockLims instance.
    # Returning teh new enteties

    @staticmethod
    def ensure_lims_process(
        lims: MockLims,
        process_id: str = "SomeID",
        process_type_name: str = "SomeTypeOfName",
        date_run: str = "2020-01-01",
        output_artifacts: list = [],
        input_artifacts: list = [],
    ) -> MockProcess:
        """Setting up a complete process with input and output artifacts."""

        process_type = MockProcessType(name=process_type_name)
        process = MockProcess(
            pid=process_id, process_type=process_type, date_run=date_run
        )
        process.input_artifact_list = input_artifacts

        lims.processes.append(process)
        lims.process_types.append(process_type)

        for artifact in output_artifacts:
            artifact.parent_process = process
            lims.artifacts.append(artifact)
        return process

    @staticmethod
    def ensure_lims_artifacts(
        lims: MockLims, artifacts_data: List[dict]
    ) -> List[MockArtifact]:
        """Creating MockArtifacts defined by <artifacts_data>. 
        Appending the new MockArtifacts to the given MockLims instance.
        Returning the new MockArtifacts"""

        artifacts = []
        for data in artifacts_data:
            artifact_data = deepcopy(data)
            if artifact_data.get("samples"):
                samples = Helpers.create_samples(artifact_data["samples"])
                artifact_data["samples"] = samples

            if artifact_data.get("parent_process"):
                process = Helpers.ensure_lims_process(
                    **artifact_data.get("parent_process")
                )
                artifact_data["parent_process"] = process
            artifact = MockArtifact(**artifact_data)

            artifacts.append(artifact)
        lims.artifacts.extend(artifacts)
        return artifacts


    # Create methods:
    # Creating and returning enteties defined by arguments. 

    @staticmethod
    def create_samples(samples_data: List[dict] = []) -> List[MockSample]:
        """Create a mock samples"""

        samples = []
        for sample_data in samples_data:
            sample = MockSample(**sample_data)
            samples.append(sample)

        return samples

    @staticmethod
    def create_artifact(samples: list = [], type: str = "") -> MockArtifact:
        """Create a mock artifact"""

        artifact = MockArtifact(samples=samples, type=type)
        return artifact

    @staticmethod
    def create_many_artifacts(
        nr_of_artifacts: int, type: str = ""
    ) -> List[MockArtifact]:
        """Create a list of mock aritfacts"""

        artifacts = []
        for i in range(nr_of_artifacts):
            artifacts.append(MockArtifact(id=i, type=type))

        return artifacts
